function EttPushThreadPrefListener(branchName, func) {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
  var branch = prefService.getBranch(branchName);
  branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.register = function() {
    branch.addObserver("", this, false);
    branch.getChildList("", { })
          .forEach(function (name) { func(branch, name); });
  };
  this.unregister = function unregister() {
    if (branch)
      branch.removeObserver("", this);
  };
  this.observe = function(subject, topic, data) {
    if (topic == "nsPref:changed")
      func(branch, data);
  };
}

var EttPushThread = {
  /*
  prefs : Components.classes["@mozilla.org/preferences-service;1"]
                 .getService(Components.interfaces.nsIPrefService)
                 .getBranch("extensions.pttpushthread.options."),
  */
  httpCheck : /(^http:\/\/)/i,
  telnetCheck : /(^telnet:\/\/)/i,
  pttCheck : /(^telnet:\/\/ptt.cc)/i,
  pttCheck2 : /(^telnet:\/\/bbs.ptt.cc)/i,
  lineLength: 54,
  prefListener: null,
  stringBundle: null,
  pushTextTemp: null,

  EttPushThreadLoad: function(_this) {
    document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", EttPushThread.contextMenuShowing, false);
    gBrowser.addEventListener("load", EttPushThread.doc_load, true);
    gBrowser.tabContainer.addEventListener("TabSelect", EttPushThread.tab_select, true);
  },

  EttPushThreadUnload: function(_this) {
    document.getElementById("contentAreaContextMenu").removeEventListener("popupshowing", EttPushThread.contextMenuShowing, false);
    gBrowser.removeEventListener("load", EttPushThread.doc_load, true);
    gBrowser.tabContainer.addEventListener("TabSelect", EttPushThread.tab_select, true);
    _this.prefListener.unregister();
  },

  createMenuItem: function(label) {
    const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    var item = document.createElementNS(XUL_NS, "menuitem"); // create a new XUL menuitem
    item.setAttribute("label", label);
    return item;
    //
  },

  tab_select: function(event) {
    if(!EttPushThread.stringBundle)
      EttPushThread.stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService).createBundle("chrome://pttpushthread/locale/pushthread.properties");

    var linkedBrowser = event.target.linkedBrowser;
    if(linkedBrowser)
    {
      var pm = linkedBrowser.contentDocument.getElementById('popup-menu');
      var input_proxy = linkedBrowser.contentDocument.getElementById("input_proxy");
      var obj1 = linkedBrowser.contentDocument.getElementById("canvas");
      var obj2 = linkedBrowser.contentDocument.getElementById("topwin");
      if(pm && pm.getAttribute("pushThread")!="1" && input_proxy && obj1 && obj2)
      {
        var item = EttPushThread.createMenuItem(EttPushThread.stringBundle.GetStringFromName("menu_pushthread"));
        var oncommand ={
          handleEvent: function(e) {
            EttPushThread.exec();
          }
        };
        item.addEventListener('command', oncommand, false);
        pm.appendChild(item);
        pm.setAttribute("pushThread", "1");
        item.setAttribute("id", "pcmanfx_menu-pushThread");
        pm.addEventListener("popupshowing", EttPushThread.pcmanfxContextMenuShowing, false);
      }
    }
  },

  doc_load: function(event) {
    if(!EttPushThread.stringBundle)
      EttPushThread.stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService).createBundle("chrome://pttpushthread/locale/pushthread.properties");

    var hWindow;
    // find the handle to the window where the event occurred
    if ( event.originalTarget && event.originalTarget.defaultView )
      hWindow = event.originalTarget.defaultView;
    else if ( event.originalTarget && event.originalTarget.contentWindow )
      hWindow = event.originalTarget.contentWindow;
    else if ( event.target && event.target.linkedBrowser && event.target.linkedBrowser.contentWindow )
      hWindow = event.target.linkedBrowser.contentWindow;

    if (gBrowser && gBrowser.currentURI){
      var urlstr = gBrowser.currentURI.spec;
      if(EttPushThread.telnetCheck.test(urlstr))
      {
        if ( hWindow && hWindow.document )
        {
          var pm = hWindow.document.getElementById("popup-menu");
          var input_proxy = hWindow.document.getElementById("input_proxy");
          var obj1 = hWindow.document.getElementById("canvas");
          var obj2 = hWindow.document.getElementById("topwin");
          if(pm && pm.getAttribute("pushThread")!="1" && input_proxy && obj1 && obj2)
          {
            var item = EttPushThread.createMenuItem(EttPushThread.stringBundle.GetStringFromName("menu_pushthread"));
            var oncommand ={
              handleEvent: function(e) {
                EttPushThread.exec();
              }
            };
            item.addEventListener('command', oncommand, false);
            pm.appendChild(item);
            pm.setAttribute("pushThread", "1");
            item.setAttribute("id", "pcmanfx_menu-pushThread");
            pm.addEventListener("popupshowing", EttPushThread.pcmanfxContextMenuShowing, false);
          }
          else
          {
            //pcmanfx-unofficial
            //var cmex = document.getElementById("contentAreaContextMenuEx");
            //if(cmex)
            //{
            //  var item = EttPushThread.createMenuItem(EttPushThread.stringBundle.GetStringFromName("menu_pushthread"));
            //  var oncommand ={
            //    handleEvent: function(e) {
            //      EttPushThread.exec();
            //    }
            //  };
            //  item.addEventListener('command', oncommand, false);
            //  item.setAttribute("id", 'bbsfox_menuEx-pushThread');
            //  cmex.appendChild(item); //only need insert once!
            //}

          }
        }
      }
      else
      {
      }
    }
  },

  pcmanfxContextMenuShowing: function(e) {
  	var menuitem = null;
  	var children = e.target.childNodes;
    for (var i = 0; i < children.length; i++) 
    {
    	if(children[i].getAttribute('id')=="pcmanfx_menu-pushThread")
        menuitem = children[i];
    };
    var urlstr = gBrowser.currentURI.spec;
    if(EttPushThread.pttCheck.test(urlstr) || EttPushThread.pttCheck2.test(urlstr))  
    	menuitem.hidden = false;
    else
      menuitem.hidden = true;
  },
  
  contextMenuShowing: function(e) {
    var _this = EttPushThread;

    var menuitem = null;
    var menuitemBBSFox = null;
    if (gBrowser && gBrowser.currentURI){
      var urlstr = gBrowser.currentURI.spec;
      if(_this.telnetCheck.test(urlstr)) //telnet ?
      {
        menuitemBBSFox = document.getElementById("bbsfox_menu-pushThread"); //if BBSFox, hide menu.
        menuitem = document.getElementById("pttpushthread_menu-pushThread");
        menuitem.hidden = menuitemBBSFox;
        return;
      }
    }
    menuitem = document.getElementById("pttpushthread_menu-pushThread");
    menuitem.hidden = true;
  },

  exec: function() {
    const EMURL = "chrome://pttpushthread/content/pushthread.xul";
    const EMFEATURES = "chrome, dialog=yes, resizable=yes, modal=yes, centerscreen";
    var retVals = { exec: false, pushText: this.pushTextTemp, lineLength: this.lineLength};
    var retVals2 = [];
    window.openDialog(EMURL, "", EMFEATURES, retVals, retVals2);
    if(!retVals.exec)
    {
      this.pushTextTemp = retVals.pushText;
    }
    else
    {
      if (gBrowser && gBrowser.currentURI){
        var urlstr = gBrowser.currentURI.spec;
        if(this.telnetCheck.test(urlstr)) //telnet ?
        {
          var cmdhandler = gBrowser.contentDocument.getElementById("cmdHandler");
          var obj1 = gBrowser.contentDocument.getElementById("main");
          var obj2 = gBrowser.contentDocument.getElementById("cursor");
          var obj3 = gBrowser.contentDocument.getElementById("hideobj");
          if(cmdhandler && obj1 && obj2 && obj3)
          {
            return;
          }
          else //other telnet add-on ?
          {
            //for telnet add-on PCMan BBS Extension 0.2.5
            var input_proxy = gBrowser.contentDocument.getElementById("input_proxy");
            var obj4 = gBrowser.contentDocument.getElementById("canvas");
            var obj5 = gBrowser.contentDocument.getElementById("topwin");
            var obj6 = gBrowser.contentDocument.getElementById("BBSWin");

            if(input_proxy && obj4 && obj5 && obj6)
            {
              for(var i =0;i<retVals2.length;++i)
              {
                input_proxy.value = retVals2[i];
                this.sendEvent(1);
                this.sendEvent(4);
                input_proxy.value = 'y';
                this.sendEvent(1);
                this.sendEvent(4);
              }
            }
            else if(input_proxy && obj4 && obj5)
            {
              for(var i =0;i<retVals2.length;++i)
              {
                input_proxy.value = retVals2[i];
                this.sendEvent(1);
                this.sendEvent(2);
                input_proxy.value = 'y';
                this.sendEvent(1);
                this.sendEvent(2);
              }
            }
            else //other telnet add-on ?
            {
              //for telnet add-on fireBBS
              var input_area = gBrowser.contentDocument.getElementById("input_area");
              var obj7 = gBrowser.contentDocument.getElementById("main");
              var obj8 = gBrowser.contentDocument.getElementById("output_area");
              var obj9 = gBrowser.contentDocument.getElementById("float_box");
              if(input_area && obj7 && obj8 && obj9)
              {
                for(var i =0;i<retVals2.length;++i)
                {
                  input_area.value = retVals2[i];
                  this.sendEvent(3);
                  this.sendEvent(2);
                  input_area.value = 'y';
                  this.sendEvent(3);
                  this.sendEvent(2);
                }

              }
            }
            return;
          }
        }
      }
      //for web page
    }
  },

  sendEvent: function(type) {
    var doc = gBrowser.contentDocument;
    if ("createEvent" in doc) {

      if(type==1) //input event to pcmanfx's textbox
      {
        var evt = doc.createEvent("UIEvents");
        var input_proxy = gBrowser.contentDocument.getElementById("input_proxy");
        evt.initUIEvent("input", true, true, document.commandDispatcher.focusedWindow, 0);
        input_proxy.dispatchEvent(evt);
      }
      else if(type==2)//keypress event to pcmanfx/fireBBS window
      {
        var evt = doc.createEvent("UIEvents");
        evt.initUIEvent("keypress", false, true, document.commandDispatcher.focusedWindow, 0);
        evt.keyCode = 13;
        evt.ctrlKey = false;
        evt.altKey = false;
        evt.shiftKey = false;
        evt.metaKey = false;
        evt.which = 13;
        evt.charCode = 0;
        document.commandDispatcher.focusedWindow.dispatchEvent(evt);
      }
      else if(type==3) //input event to fireBBS's input_area
      {
        var evt = doc.createEvent("UIEvents");
        evt.initUIEvent("compositionend", true, true, document.commandDispatcher.focusedWindow, 0);
        doc.dispatchEvent(evt);
      }
      else if(type==4)//keypress event to pcmanfx/fireBBS window
      {
        var evt = doc.createEvent("UIEvents");
        var input_proxy = gBrowser.contentDocument.getElementById("input_proxy");
        evt.initUIEvent("keypress", false, true, document.commandDispatcher.focusedWindow, 0);
        evt.keyCode = 13;
        evt.ctrlKey = false;
        evt.altKey = false;
        evt.shiftKey = false;
        evt.metaKey = false;
        evt.which = 13;
        evt.charCode = 0;
        input_proxy.dispatchEvent(evt);
      }
    }
  },

  onPrefChange: function(_this, branch, name) {
    try {
      switch (name) {
        case "lineLength":
          _this.lineLength = branch.getIntPref(name);
          break;
      }
    } catch(e) {
      // eats all errors
      return;
    }
  }

};

EttPushThread.prefListener = new EttPushThreadPrefListener('extensions.pttpushthread.options.', function(branch, name) {EttPushThread.onPrefChange(EttPushThread, branch, name);});
EttPushThread.prefListener.register();

window.addEventListener("load",   function() { EttPushThread.EttPushThreadLoad(EttPushThread); },   false);
window.addEventListener("unload", function() { EttPushThread.EttPushThreadUnload(EttPushThread); }, false);